Deckstravaganza = Deckstravaganza or class()

Deckstravaganza.RequireHookFiles = {
  "lib/managers/menu/playerinventorygui",
  "lib/managers/menu/skilltreegui",
  "lib/managers/multiprofilemanager",
  "lib/managers/skilltreemanager",
  "lib/setups/setup"
}

Deckstravaganza.VERSION = 10

local function contains(tab, val)
  if tab then
    for _, v in pairs(tab) do
      if v == val then
        return true
      end
    end
  end
  return false
end

local function perk_equals(p1, p2)
  local res = true
  for k, v in pairs(p1) do
    res = res and v == p2[k]
  end
  return res
end

local function perk_contains(tab, val)
  for _, v in pairs(tab) do
    if perk_equals(v, val) then
      return true
    end
  end
  return false
end

local function disable_deckstravaganza()
  managers.deckstravaganza._global.active = false
end

local exit_button = { text = "Exit", callback = disable_deckstravaganza }

function Deckstravaganza:init()
  self:_setup()
  return self
end

function Deckstravaganza:_setup(reset)
  local deck_nums = {
    crew_chief = 1,
    muscle = 2,
    armorer = 3,
    rogue = 4,
    hitman = 5,
    crook = 6,
    burglar = 7,
    infiltrator = 8,
    sociopath = 9,
    gambler = 10,
    grinder = 11,
    yakuza = 12,
    ex_president = 13,
    maniac = 14,
    anarchist = 15,
    biker = 16,
    kingpin = 17,
    sicario = 18,
    stoic = 19,
    tag_team = 20,
    hacker = 21,
    leech = 22
  }
  local deck_names = {
    "crew_chief",
    "muscle",
    "armorer",
    "rogue",
    "hitman",
    "crook",
    "burglar",
    "infiltrator",
    "sociopath",
    "gambler",
    "grinder",
    "yakuza",
    "ex_president",
    "maniac",
    "anarchist",
    "biker",
    "kingpin",
    "sicario",
    "stoic",
    "tag_team",
    "hacker",
    "leech"
  }
  local prereqs = {
    {
      perk = { "muscle", 3 },
      reqs = {
        { "crew_chief", 5 },
        { "muscle", 1 },
        { "gambler", 3 },
        { "grinder", 3 },
        { "ex_president", 3 },
        { "kingpin", 3 },
        { "tag_team", 3 },
        { "hacker", 3 }
      }
    },
    {
      perk = { "muscle", 5 },
      reqs = {
        { "muscle", 3 },
        { "gambler", 3 },
        { "grinder", 3 },
        { "ex_president", 5 },
        { "kingpin", 5 },
        { "tag_team", 3 },
        { "hacker", 3 },
        { "leech", 3 }
      }
    },
    {
      perk = { "muscle", 9 },
      reqs = {
        { "muscle", 5 },
        { "gambler", 5 },
        { "grinder", 7 },
        { "ex_president", 7 },
        { "kingpin", 7 },
        { "tag_team", 7 },
        { "leech", 7 }
      }
    },
    {
      perk = { "armorer", 3 },
      reqs = {
        { "armorer", 1 },
        { "sociopath", 3 }
      }
    },
    {
      perk = { "armorer", 5 },
      reqs = {
        { "armorer", 3 },
        { "sociopath", 7 }
      }
    },
    {
      perk = { "armorer", 9 },
      reqs = { { "armorer", 5 } }
    },
    {
      perk = { "rogue", 5 },
      reqs = {
        { "rogue", 1 },
        { "crook", 1 },
        { "ex_president", 5 },
        { "sicario", 5 },
        { "hacker", 5 }
      }
    },
    {
      perk = { "rogue", 7 },
      reqs = {
        { "rogue", 5 },
        { "hacker", 9 }
      }
    },
    {
      perk = { "hitman", 3 },
      reqs = { { "hitman", 1 } }
    },
    {
      perk = { "hitman", 5 },
      reqs = { { "hitman", 3 } }
    },
    {
      perk = { "hitman", 7 },
      reqs = { { "hitman", 5 } }
    },
    {
      perk = { "hitman", 9 },
      reqs = { { "hitman", 7 } }
    },
    {
      perk = { "crook", 5 },
      reqs = { { "crook", 3 } }
    },
    {
      perk = { "crook", 7 },
      reqs = { { "crook", 5 } }
    },
    {
      perk = { "burglar", 3 },
      reqs = { { "burglar", 1 } }
    },
    {
      perk = { "burglar", 5 },
      reqs = { { "burglar", 3 } }
    },
    {
      perk = { "burglar", 7 },
      reqs = { { "burglar", 5 } }
    },
    {
      perk = { "infiltrator", 5 },
      reqs = { { "infiltrator", 3 } }
    },
    {
      perk = { "infiltrator", 7 },
      reqs = { { "infiltrator", 5 } }
    },
    {
      perk = { "sociopath", 7 },
      reqs = {
        { "armorer", 1 },
        { "sociopath", 3 }
      }
    },
    {
      perk = { "gambler", 3 },
      reqs = { { "gambler", 1 } }
    },
    {
      perk = { "gambler", 5 },
      reqs = { { "gambler", 3 } }
    },
    {
      perk = { "gambler", 7 },
      reqs = { { "gambler", 1 } }
    },
    {
      perk = { "gambler", 9 },
      reqs = { { "gambler", 7 } }
    },
    {
      perk = { "grinder", 3 },
      reqs = { { "grinder", 1 } }
    },
    {
      perk = { "grinder", 5 },
      reqs = { { "grinder", 3 } }
    },
    {
      perk = { "grinder", 7 },
      reqs = { { "grinder", 5 } }
    },
    {
      perk = { "grinder", 9 },
      reqs = { { "grinder", 5 } }
    },
    {
      perk = { "yakuza", 5 },
      reqs = { { "yakuza", 1 } }
    },
    {
      perk = { "yakuza", 7 },
      reqs = { { "yakuza", 5 } }
    },
    {
      perk = { "yakuza", 9 },
      reqs = {
        { "yakuza", 1 },
        { "yakuza", 3 }
      }
    },
    {
      perk = { "ex_president", 3 },
      reqs = { { "ex_president", 1 } }
    },
    {
      perk = { "ex_president", 5 },
      reqs = { { "ex_president", 3 } }
    },
    {
      perk = { "ex_president", 7 },
      reqs = { { "ex_president", 5 } }
    },
    {
      perk = { "maniac", 3 },
      reqs = { { "maniac", 1 } }
    },
    {
      perk = { "maniac", 5 },
      reqs = { { "maniac", 1 } }
    },
    {
      perk = { "maniac", 7 },
      reqs = { { "maniac", 1 } }
    },
    {
      perk = { "maniac", 9 },
      reqs = { { "maniac", 1 } }
    },
    {
      perk = { "anarchist", 5 },
      reqs = { { "anarchist", 3 } }
    },
    {
      perk = { "anarchist", 7 },
      reqs = { { "anarchist", 5 } }
    },
    {
      perk = { "biker", 3 },
      reqs = { { "biker", 1 } }
    },
    {
      perk = { "biker", 5 },
      reqs = { { "biker", 1 } }
    },
    {
      perk = { "biker", 7 },
      reqs = { { "biker", 1 } }
    },
    {
      perk = { "biker", 9 },
      reqs = { { "biker", 1 } }
    },
    {
      perk = { "kingpin", 3 },
      reqs = { { "kingpin", 1 } }
    },
    {
      perk = { "kingpin", 5 },
      reqs = { { "kingpin", 3 } }
    },
    {
      perk = { "kingpin", 7 },
      reqs = { { "kingpin", 5 } }
    },
    {
      perk = { "kingpin", 9 },
      reqs = { { "kingpin", 7 } }
    },
    {
      perk = { "sicario", 9 },
      reqs = { { "sicario", 1 } }
    },
    {
      perk = { "stoic", 5 },
      reqs = { { "stoic", 1 } }
    },
    {
      perk = { "stoic", 7 },
      reqs = { { "stoic", 1 } }
    },
    {
      perk = { "stoic", 9 },
      reqs = { { "stoic", 1 } }
    },
    {
      perk = { "tag_team", 5 },
      reqs = { { "tag_team", 1 } }
    },
    {
      perk = { "tag_team", 7 },
      reqs = {
        { "muscle", 3 },
        { "gambler", 3 },
        { "grinder", 3 },
        { "ex_president", 5 },
        { "kingpin", 5 },
        { "tag_team", 3 },
        { "hacker", 3 },
        { "leech", 3 }
      }
    },
    {
      perk = { "tag_team", 9 },
      reqs = { { "tag_team", 1 } }
    },
    {
      perk = { "hacker", 5 },
      reqs = { { "hacker", 1 } }
    },
    {
      perk = { "hacker", 7 },
      reqs = { { "hacker", 1 } }
    },
    {
      perk = { "hacker", 9 },
      reqs = { { "hacker", 5 } }
    },
    {
      perk = { "leech", 3 },
      reqs = { { "leech", 1 } }
    },
    {
      perk = { "leech", 5 },
      reqs = { { "leech", 1 } }
    },
    {
      perk = { "leech", 7 },
      reqs = { { "leech", 3 } }
    },
    {
      perk = { "leech", 9 },
      reqs = { { "leech", 7 } }
    }
  }
  local num_prereqs = {}
  for num, _ in pairs(tweak_data.skilltree.specializations) do
    local deck_reqs = {}
    for i = 1, 9 do
      deck_reqs[i] = {}
    end
    for _, prereq in pairs(prereqs) do
      if prereq.perk[1] == deck_names[num] then
        local newreqs = {}
        for _, req in pairs(prereq.reqs) do
          table.insert(newreqs, { deck_nums[req[1]], req[2] })
        end
        deck_reqs[prereq.perk[2]] = newreqs
      end
    end
    num_prereqs[num] = deck_reqs
  end
  if not Global.deckstravaganza_manager or reset then
    Global.deckstravaganza_manager = {
      VERSION = Deckstravaganza.VERSION,
      prereqs = num_prereqs,
      profiles = {},
      deck_one = nil,
      one_name = "None",
      deck_two = nil,
      two_name = "None",
      enabled = false,
      custom_deck = {},
      custom_prev = nil,
      changed = false,
      active = false
    }
  end
  self._global = Global.deckstravaganza_manager
end

function Deckstravaganza:save(data)
  local state = {
    VERSION = self._global.VERSION or 0,
    profiles = self._global.profiles,
    deck_one = self._global.deck_one,
    one_name = self._global.one_name,
    deck_two = self._global.deck_two,
    two_name = self._global.two_name,
    enabled = self._global.enabled,
    custom_deck = self._global.custom_deck,
  }
  data.Deckstravaganza = state
end

function Deckstravaganza:load(data, version)
  local state = data.Deckstravaganza
  if not state or not state.VERSION or state.VERSION ~= Deckstravaganza.VERSION then
    self:_setup(true)
  else
    self._global.VERSION = state.VERSION
    self._global.profiles = state.profiles or {}
    self._global.deck_one = state.deck_one or self._global.deck_one
    self._global.one_name = state.one_name or self._global.one_name
    self._global.deck_two = state.deck_two or self._global.deck_two
    self._global.two_name = state.two_name or self._global.two_name
    self._global.enabled = state.enabled or self._global.enabled
    self._global.custom_deck = state.custom_deck or {}
  end
  self._global.custom_prev = nil
  self._global.changed = false
  self:_preload_decks()
end

function Deckstravaganza:save_profile(index)
  local state = {
    deck_one = self._global.deck_one,
    one_name = self._global.one_name,
    deck_two = self._global.deck_two,
    two_name = self._global.two_name,
    enabled = self._global.enabled,
    custom_deck = self._global.custom_deck
  }
  self._global.profiles[index] = state
end

function Deckstravaganza:load_profile(index)
  if self._global.deck_one then
    self.unequip_deck({ self, self._global.deck_one })
  end
  if self._global.deck_two then
    self.unequip_deck({ self, self._global.deck_two })
  end
  local state = self._global.profiles[index]
  if state then
    self._global.deck_one = state.deck_one
    self._global.one_name = state.one_name
    self._global.deck_two = state.deck_two
    self._global.two_name = state.two_name
    self._global.enabled = state.enabled
    self._global.custom_deck = state.custom_deck
  else
    self._global.deck_one = nil
    self._global.one_name = "None"
    self._global.deck_two = nil
    self._global.two_name = "None"
    self._global.enabled = false
    self._global.custom_deck = {}
  end
  self._global.custom_prev = nil
  self._global.changed = false
  self:_preload_decks()
end

function Deckstravaganza:_preload_decks()
  local cur_spec = self:get_current_specialization()
  self.unequip_deck({ self, cur_spec })
  if self._global.enabled then
    if self._global.deck_one then
      if self._global.deck_one == "custom" then
        self:apply_deck()
        self._global.one_name = "Custom"
      else
        self.equip_deck({ self, 1, self._global.deck_one })
        self._global.one_name = self:get_perk_deck_name(self._global.deck_one)
      end
    end
  else
    self.equip_deck({ self, 1, cur_spec })
    self._global.deck_one = cur_spec
    self._global.one_name = self:get_perk_deck_name(cur_spec)
    self._global.enabled = false
  end
  if self._global.deck_two then
    self.equip_deck({ self, 2, self._global.deck_two })
    self._global.two_name = self:get_perk_deck_name(self._global.deck_two)
  else
    self._global.deck_two = nil
    self._global.two_name = "None"
  end
end

function Deckstravaganza:get_current_specialization()
  return managers.skilltree:get_specialization_value("current_specialization")
end

function Deckstravaganza:get_perk_deck_name(deck)
  return managers.localization:text(tweak_data.skilltree.specializations[deck].name_id, nil)
end

function Deckstravaganza:get_custom_perk(card)
  local deck = self._global.custom_deck[card][1]
  local perk = self._global.custom_deck[card][2]
  return deck, perk
end

function Deckstravaganza.unequip_deck(data)
  local self = data[1]
  local deck = data[2]
  local perk, cdeck, cperk, ccard
  if deck == "custom" then
    cdeck = self._global.custom_deck
    if self._global.changed then cdeck = self._global.custom_prev end
    for card = 1, 9 do
      if cdeck[card] then
        cperk, ccard = self:get_custom_perk(card)
        perk = tweak_data.skilltree.specializations[cperk][ccard]
        for _, upgrade in pairs(perk.upgrades) do
          if managers.upgrades:aquired(upgrade) then
            managers.upgrades:unaquire(upgrade)
          end
        end
      end
    end
  else
    for card = 1, 9 do
      perk = tweak_data.skilltree.specializations[deck][card]
      for _, upgrade in pairs(perk.upgrades) do
        if managers.upgrades:aquired(upgrade) then
          managers.upgrades:unaquire(upgrade, "SpecializationTree" .. tostring(deck))
          managers.upgrades:unaquire(upgrade)
        end
      end
    end
  end
end

function Deckstravaganza.reset_decks(data)
  local function reset_deck(self, slot)
    local cur_spec = self:get_current_specialization()
    if slot == 1 then
      if self._global.deck_one then
        self.unequip_deck({ self, self._global.deck_one })
      end
      self.equip_deck({ self, 1, cur_spec })
      self._global.deck_one = cur_spec
      self._global.one_name = self:get_perk_deck_name(cur_spec)
      self._global.enabled = false
    elseif slot == 2 then
      if self._global.deck_two then
        self.unequip_deck({ self, self._global.deck_two })
      end
      self._global.deck_two = nil
      self._global.two_name = "None"
    end
  end
  
  local self = data[1]
  local slot = data[2]
  if slot == "custom" then slot = 1 end
  if slot then
    reset_deck(self, slot)
  else
    reset_deck(self, 1)
    reset_deck(self, 2)
  end
  self._global.active = false
end

function Deckstravaganza.equip_deck(data)
  local self = data[1]
  local slot = data[2]
  local deck = data[3]
  if deck == "custom" then
    self:apply_deck()
    return
  end
  local perk, card
  if slot == 1 then
    if self._global.deck_one and self._global.deck_one ~= deck then
      self.unequip_deck({ self, self._global.deck_one })
    end
    self._global.deck_one = deck
    self._global.one_name = self:get_perk_deck_name(deck)
    self._global.enabled = true
  else
    if self._global.deck_two and self._global.deck_two ~= deck then
      self.unequip_deck({ self, self._global.deck_two })
    end
    self._global.deck_two = deck
    self._global.two_name = self:get_perk_deck_name(deck)
  end
  if self._global.deck_one == self._global.deck_two then
    self._global.deck_two = nil
    self._global.two_name = "None"
  end
  for card = 1, 9 do
    perk = tweak_data.skilltree.specializations[deck][card]
    for _, upgrade in pairs(perk.upgrades) do
      if not managers.upgrades:aquired(upgrade) then
        managers.upgrades:aquire(upgrade, true)
      end
    end
  end
  self._global.active = false
end

function Deckstravaganza.double_deck(data)
  local self = data[1]
  local slot = data[2]
  local double_deck_options = {}
  local title, deck, elem, double_deck_menu
  if slot == 1 then
    title = "Primary Deck: " .. self._global.one_name
  else
    title = "Secondary Deck: " .. self._global.two_name
  end
  for deck = 1, #tweak_data.skilltree.specializations do
    elem = { text = self:get_perk_deck_name(deck), callback = self.equip_deck, data = { self, slot, deck } }
    table.insert(double_deck_options, elem)
  end
  table.insert(double_deck_options, {})
  table.insert(double_deck_options, { text = "Unequip", callback = self.reset_decks, data = { self, slot } })
  table.insert(double_deck_options, { text = "Back", callback = self.deckstravaganza, data = self })
  table.insert(double_deck_options, exit_button)
  self._global.active = true
  double_deck_menu = SimpleMenu:new(title, "Choose a deck.", double_deck_options)
  double_deck_menu:show()
end

function Deckstravaganza.apply_deck(self)
  local deck, card, perk
  self.unequip_deck({ self, self._global.deck_one })
  for card = 1, 9 do
    if self._global.custom_deck[card] then
      deck, perk = self:get_custom_perk(card)
      perk = tweak_data.skilltree.specializations[deck][perk]
      for _, upgrade in pairs(perk.upgrades) do
        if not managers.upgrades:aquired(upgrade) then
          managers.upgrades:aquire(upgrade, true)
        end
      end
    end
  end
  self._global.deck_one = "custom"
  self._global.one_name = "Custom"
  self._global.changed = false
  self._global.custom_prev = nil
  self._global.enabled = true
  self._global.active = false
end

function Deckstravaganza.clear_deck(self)
  if not self._global.custom_prev then self._global.custom_prev = self._global.custom_deck end
  for card = 1, 9 do
    self._global.custom_deck[card] = nil
  end
  self._global.changed = true
  self._global.active = false
end

function Deckstravaganza:get_prereqs(card)
  local deck = card[1]
  local perk = card[2]
  local prereqs = self._global.prereqs
  local reqs
  if prereqs[deck] then
    reqs = prereqs[deck][perk]
    if reqs and next(reqs) ~= nil then
      return prereqs[deck][perk]
    else
      return nil
    end
  else
    if perk == 1 or math.mod(perk, 2) == 0 then
      return nil
    else
      return { { deck, perk - 2 } }
    end
  end
end

function Deckstravaganza.remove_card_chain(data)
  local function rem_chain(data)
    local self = data[1]
    local chain = data[2]
    if not self._global.custom_prev then self._global.custom_prev = self._global.custom_deck end
    for _, k in pairs(chain) do
      self._global.custom_deck[k] = nil
    end
    self._global.changed = true
    self._global.active = false
  end
  
  local function rem_card(data)
    local self = data[1]
    local chain = data[2]
    local card = chain[1]
    local cdeck = self._global.custom_deck
    local is_prereq_options = {}
    local req_num, req_card, reqs, is_req, deck, perk, is_prereq_menu
    for k = 1, 9 do
      if k ~= card then
        req_num = k
        req_card = cdeck[k]
        if req_card then
          reqs = self:get_prereqs(req_card)
          if reqs then
            for j, prereq in pairs(reqs) do
              if perk_equals(cdeck[card], prereq) then
                is_req = true
                table.insert(chain, 1, req_num)
                break
              end
            end
          end
        end
      end
    end
    if is_req then
      rem_card({ self, chain })
    else
      for _, k in pairs(chain) do
        deck, perk = self:get_custom_perk(k)
        table.insert(is_prereq_options, 1, {
          text = "Card " .. tostring(k) .. ": " .. self:get_perk_deck_name(deck) .. " " .. tostring(perk),
          callback = self.desc_perk,
          data = { self, k, deck, perk, "none", "remove_card_chain", { self, chain[#chain] } }
        })
      end
      table.insert(is_prereq_options, {})
      table.insert(is_prereq_options, { text = "Remove All", callback = rem_chain, data = { self, chain } })
      table.insert(is_prereq_options, exit_button)
      if #chain > 1 then
        self._global.active = true
        is_prereq_menu = SimpleMenu:new("Remove Cards", "Card " .. tostring(chain[#chain]) .. " is a prerequisite for other cards in your Custom Deck.\nThe following cards will be removed:", is_prereq_options)
        is_prereq_menu:show()
      else
        rem_chain({ self, chain })
      end
    end
  end
  
  local self = data[1]
  local chain = { data[2] }
  rem_card({ self, chain })
end

function Deckstravaganza.set_perk(data)
  local self = data[1]
  local card = data[2]
  local deck = data[3]
  local perk = data[4]
  local elem = { deck, perk }
  if perk_contains(self._global.custom_deck, elem) then
    local dupe_menu = SimpleMenu:new("Duplicate card",
      self:get_perk_deck_name(deck) .. " " .. tostring(perk) .. "is already in your deck.",
      {
        {},
        { text = "Back", callback = self.pick_perk, data = { self, card, deck } },
        exit_button
      })
    self._global.active = true
    dupe_menu:show()
    return
  end
  if self._global.custom_deck[card] then
    self.remove_card_chain({ self, card })
  end
  local no_prereq_options = {}
  local reqs, need_req, have_req, rdeck, rperk, no_prereq_menu
  reqs = self:get_prereqs(elem)
  if reqs then
    need_req = true
    for j, prereq in pairs(reqs) do
      rdeck = prereq[1]
      rperk = prereq[2]
      table.insert(no_prereq_options, {
        text = self:get_perk_deck_name(rdeck) .. " " .. tostring(rperk),
        callback = self.desc_perk,
        data = { self, card, rdeck, rperk, "set_perk", "set_perk", data }
      })
      if perk_contains(self._global.custom_deck, prereq) then
        have_req = true
        break
      end
    end
  end
  if have_req or not need_req then
    if not self._global.custom_prev then self._global.custom_prev = self._global.custom_deck end
    self._global.custom_deck[card] = elem
    self._global.changed = true
    self:pick_card()
  else
    table.insert(no_prereq_options, {})
    table.insert(no_prereq_options, { text = "Back", callback = self.pick_perk, data = { self, card, deck } })
    table.insert(no_prereq_options, exit_button)
    self._global.active = true
    no_prereq_menu = SimpleMenu:new("Missing prerequisite", "Choose one of the following prerequisite options to replace card " .. tostring(card) .. ".", no_prereq_options)
    no_prereq_menu:show()
  end
end

function Deckstravaganza.desc_perk(data)
  local self = data[1]
  local card = data[2]
  local deck = data[3]
  local perk = data[4]
  local oper = data[5]
  local back = data[6]
  local back_data = data[7]
  local desc_perk_options = {}
  local desc_perk_menu
  local specialization_descs = tweak_data.upgrades.specialization_descs[deck]
  specialization_descs = specialization_descs and specialization_descs[perk] or {}
  local macroes = {}
  for i, d in pairs(specialization_descs) do
    macroes[i] = d
  end
  local tier_data = tweak_data.skilltree.specializations[deck][perk]
  local name_string = tier_data.name_id and managers.localization:text(tier_data.name_id) or "NO_NAME_" .. tostring(deck) .. "_" .. tostring(perk)
  local desc_string = tier_data.desc_id and managers.localization:text(tier_data.desc_id, macroes) or "NO_DESC_" .. tostring(deck) .. "_" .. tostring(perk)
  desc_string = string.gsub(desc_string, "##", "")
  local opt_elem = { text = "", callback = self[oper], data = {} }
  if oper == "set_perk" then
    opt_elem.text = "Set as card " .. tostring(card)
    opt_elem.data = { self, card, deck, perk }
  elseif oper == "pick_deck" then
    opt_elem.text = "Pick new card"
    opt_elem.data = { self, card }
  elseif oper == "remove_card_chain" then
    opt_elem.text = "Remove Card"
    opt_elem.data = { self, card }
  else
    opt_elem = {}
  end
  table.insert(desc_perk_options, {})
  table.insert(desc_perk_options, opt_elem)
  table.insert(desc_perk_options, { text = "Back", callback = self[back], data = back_data })
  table.insert(desc_perk_options, exit_button)
  self._global.active = true
  desc_perk_menu = SimpleMenu:new(name_string, desc_string, desc_perk_options)
  desc_perk_menu:show()
end

function Deckstravaganza.pick_perk(data)
  local self = data[1]
  local card = data[2]
  local deck = data[3]
  local pick_perk_options = {}
  local elem, pick_perk_menu
  for perk = 1, 9 do
    elem = {
      text = tostring(perk) .. " " .. managers.localization:text(tweak_data.skilltree.specializations[deck][perk].name_id, nil),
      callback = self.desc_perk,
      data = { self, card, deck, perk, "set_perk", "pick_perk", data }
    }
    table.insert(pick_perk_options, elem)
  end
  table.insert(pick_perk_options, {})
  table.insert(pick_perk_options, { text = "Back", callback = self.pick_deck, data = { self, card } })
  table.insert(pick_perk_options, exit_button)
  self._global.active = true
  pick_perk_menu = SimpleMenu:new("Custom Card " .. card .. ": " .. self:get_perk_deck_name(deck), "Choose a perk.", pick_perk_options)
  pick_perk_menu:show()
end

function Deckstravaganza.pick_deck(data)
  local self = data[1]
  local card = data[2]
  local pick_deck_options = {}
  local elem, pick_deck_menu
  for deck = 1, #tweak_data.skilltree.specializations do
    elem = {
      text = self:get_perk_deck_name(deck),
      callback = self.pick_perk,
      data = { self, card, deck }
    }
    table.insert(pick_deck_options, elem)
  end
  table.insert(pick_deck_options, {})
  table.insert(pick_deck_options, { text = "Back", callback = self.pick_card, data = self })
  table.insert(pick_deck_options, exit_button)
  self._global.active = true
  pick_deck_menu = SimpleMenu:new("Custom Card " .. card, "Choose a deck.", pick_deck_options)
  pick_deck_menu:show()
end

function Deckstravaganza.pick_card(self)
  local cdeck = self._global.custom_deck
  local pick_card_options = {}
  local opt_text, deck, perk, elem, pick_card_menu
  for card = 1, 9 do
    opt_text = "Card " .. tostring(card) .. ": "
    if cdeck and cdeck[card] then
      deck, perk = self:get_custom_perk(card)
      opt_text = opt_text .. self:get_perk_deck_name(deck) .. " " .. tostring(perk)
      elem = {
        text = opt_text,
        callback = self.desc_perk,
        data = { self, card, deck, perk, "remove_card_chain", "pick_card", self }
      }
    else
      elem = {
        text = opt_text,
        callback = self.pick_deck,
        data = { self, card }
      }
    end
    table.insert(pick_card_options, elem)
  end
  table.insert(pick_card_options, {})
  table.insert(pick_card_options, { text = "Equip", callback = self.apply_deck, data = self })
  table.insert(pick_card_options, { text = "Unequip", callback = self.reset_decks, data = { self, "custom" } })
  table.insert(pick_card_options, {})
  table.insert(pick_card_options, { text = "Clear", callback = self.clear_deck, data = self })
  table.insert(pick_card_options, { text = "Back", callback = self.deckstravaganza, data = self })
  table.insert(pick_card_options, exit_button)
  self._global.active = true
  pick_card_menu = SimpleMenu:new("Custom Deck", "Choose a card.", pick_card_options)
  pick_card_menu:show()
end

function Deckstravaganza.deckstravaganza(self)
  if not self._global.deck_one then
    self._global.deck_one = self:get_current_specialization()
    self._global.one_name = self:get_perk_deck_name(self._global.deck_one)
  end
  local deckstravaganza_options = {}
  local deckstravaganza_menu
  deckstravaganza_options = {
    { text = "Primary Deck: " .. self._global.one_name, callback = self.double_deck, data = { self, 1 } },
    { text = "Secondary Deck: " .. self._global.two_name, callback = self.double_deck, data = { self, 2 } },
    {},
    { text = "Custom Deck", callback = self.pick_card, data = self },
    {},
    { text = "Reset Decks", callback = self.reset_decks, data = { self } },
    exit_button
  }
  self._global.active = true
  deckstravaganza_menu = SimpleMenu:new("Deckstravaganza", "Customize Perk Decks", deckstravaganza_options)
  deckstravaganza_menu:show()
end

if RequiredScript == "lib/setups/setup" then
  Hooks:PostHook(Setup, "init_managers", "InitDeckstravaganza", function()
    managers.deckstravaganza = Deckstravaganza:new()
  end)
end

if RequiredScript == "lib/managers/skilltreemanager" then
  if not _SkillTreeManager_set_current_specialization then _SkillTreeManager_set_current_specialization = SkillTreeManager.set_current_specialization end
  function SkillTreeManager:set_current_specialization(tree)
    local cdm = managers.deckstravaganza
    if cdm._global.deck_one then
      cdm.unequip_deck({ cdm, cdm._global.deck_one })
    end
    if cdm._global.deck_two then
      cdm.unequip_deck({ cdm, cdm._global.deck_two })
      if cdm._global.deck_two == tree then
        cdm._global.deck_two = nil
        cdm._global.two_name = "None"
      end
    end
    local res = _SkillTreeManager_set_current_specialization(self, tree)
    cdm._preload_decks(cdm)
    if res then return true end
  end
  
  Hooks:PostHook(SkillTreeManager, "save", "SkillTreeManagerSaveDeckstravaganza", function(stm, data)
    managers.deckstravaganza.save(managers.deckstravaganza, data)
  end)
  
  Hooks:PostHook(SkillTreeManager, "load", "SkillTreeManagerLoadDeckstravaganza", function(stm, data, version)
    managers.deckstravaganza.load(managers.deckstravaganza, data, version)
  end)
end

if RequiredScript == "lib/managers/menu/playerinventorygui" then
  Hooks:PreHook(PlayerInventoryGui, "next_specialization", "PlayerInventoryGuiDisableDecksNext", function(pig)
    managers.deckstravaganza._global.enabled = false
  end)
  
  Hooks:PreHook(PlayerInventoryGui, "previous_specialization", "PlayerInventoryGuiDisableDecksPrevious", function(pig)
    managers.deckstravaganza._global.enabled = false
  end)
end

if RequiredScript == "lib/managers/menu/skilltreegui" then
  Hooks:PreHook(SkillTreeGui, "activate_specialization", "SkillTreeGuiDisableDecks", function(stg, tree, tier)
    managers.deckstravaganza._global.enabled = false
  end)
end

if RequiredScript == "lib/managers/multiprofilemanager" then
  function MultiProfileManager:set_current_profile(index)
    if index < 0 or index > self:profile_count() then
      return
    end
    if index == self._global._current_profile then
      return
    end
    local cdm = managers.deckstravaganza
    self:save_current()
    cdm.save_profile(cdm, self._global._current_profile)
    self._global._current_profile = index
    self:load_current()
    cdm.load_profile(cdm, self._global._current_profile)
    local mcm = managers.menu_component
    if mcm._player_inventory_gui then
      local node = mcm._player_inventory_gui._node
      mcm:close_inventory_gui()
      mcm:create_inventory_gui(node)
    end
    print("[MultiProfileManager:set_current_profile] current profile:", self._global._current_profile)
  end
end
